@@ -21,26 +21,14 @@ def send_pushbullet_push(token: str, message_title: str, message_body: str, verb
         headers = {"Authorization": "Bearer " + token, "Content-Type": "application/json"}
         message = {"type": "note", "title": message_title, "body": message_body}
         url = "https://api.pushbullet.com/v2/pushes"
-
-        # work out if there is an existing, identical push
-        resp = requests.get(url, headers=headers)
-        have_already_sent = False
-        for p in resp.json().get("pushes"):
-            if p.get("title") == message.get("title") and p.get("body") == message.get("body"):
-                have_already_sent = True
-                break
-
-        # if not, send one
-        if not have_already_sent:
-            if verbose:
-                logger.info("sending notification")
-            resp = requests.post(url, data=json.dumps(message), headers=headers)
-            if resp.status_code != 200:
-                logger.error(f"error {resp.status_code} sending pushbullet notification: {resp.content}")
-                return False
-            else:
-                return True
-        return True
+        if verbose:
+            logger.info("sending notification")
+        resp = requests.post(url, data=json.dumps(message), headers=headers)
+        if resp.status_code != 200:
+            logger.error(f"error {resp.status_code} sending pushbullet notification: {resp.content}")
+            return False
+        else:
+            return True
     except:
         # TODO: what type of exception is thrown here ?
         logger.error(f"Exception sending pushbullet push", exc_info=True)
